@@ -18,10 +18,7 @@ def show_toolbar(request):
     """
     Default function to determine whether to show the toolbar on a given page.
     """
-    if request.META.get("REMOTE_ADDR", None) not in settings.INTERNAL_IPS:
-        return False
-
-    return bool(settings.DEBUG)
+    return settings.DEBUG and request.META.get("REMOTE_ADDR") in settings.INTERNAL_IPS
 
 
 @lru_cache()
@@ -45,10 +42,9 @@ class DebugToolbarMiddleware:
         self.get_response = get_response
 
     def __call__(self, request):
-        # Decide whether the toolbar is active for this request. Don't render
-        # the toolbar during AJAX requests.
+        # Decide whether the toolbar is active for this request.
         show_toolbar = get_show_toolbar()
-        if not show_toolbar(request) or request.is_ajax():
+        if not show_toolbar(request) or DebugToolbar.is_toolbar_request(request):
             return self.get_response(request)
 
         toolbar = DebugToolbar(request, self.get_response)
@@ -65,60 +61,48 @@ class DebugToolbarMiddleware:
             for panel in reversed(toolbar.enabled_panels):
                 panel.disable_instrumentation()
 
+        # Generate the stats for all requests when the toolbar is being shown,
+        # but not necessarily inserted.
+        for panel in reversed(toolbar.enabled_panels):
+            panel.generate_stats(request, response)
+            panel.generate_server_timing(request, response)
+
+        # Always render the toolbar for the history panel, even if it is not
+        # included in the response.
+        rendered = toolbar.render_toolbar()
+
+        for header, value in self.get_headers(request, toolbar.enabled_panels).items():
+            response.headers[header] = value
+
         # Check for responses where the toolbar can't be inserted.
         content_encoding = response.get("Content-Encoding", "")
         content_type = response.get("Content-Type", "").split(";")[0]
-        if any(
-            (
-                getattr(response, "streaming", False),
-                "gzip" in content_encoding,
-                content_type not in _HTML_TYPES,
-            )
+        if (
+            getattr(response, "streaming", False)
+            or content_encoding != ""
+            or content_type not in _HTML_TYPES
         ):
             return response
 
-        # Collapse the toolbar by default if SHOW_COLLAPSED is set.
-        if toolbar.config["SHOW_COLLAPSED"] and "djdt" not in request.COOKIES:
-            response.set_cookie("djdt", "hide", 864000)
-
         # Insert the toolbar in the response.
         content = response.content.decode(response.charset)
         insert_before = dt_settings.get_config()["INSERT_BEFORE"]
         pattern = re.escape(insert_before)
         bits = re.split(pattern, content, flags=re.IGNORECASE)
         if len(bits) > 1:
-            # When the toolbar will be inserted for sure, generate the stats.
-            for panel in reversed(toolbar.enabled_panels):
-                panel.generate_stats(request, response)
-                panel.generate_server_timing(request, response)
-
-            response = self.generate_server_timing_header(
-                response, toolbar.enabled_panels
-            )
-
-            bits[-2] += toolbar.render_toolbar()
+            bits[-2] += rendered
             response.content = insert_before.join(bits)
-            if response.get("Content-Length", None):
+            if "Content-Length" in response:
                 response["Content-Length"] = len(response.content)
         return response
 
     @staticmethod
-    def generate_server_timing_header(response, panels):
-        data = []
-
+    def get_headers(request, panels):
+        headers = {}
         for panel in panels:
-            stats = panel.get_server_timing_stats()
-            if not stats:
-                continue
-
-            for key, record in stats.items():
-                # example: `SQLPanel_sql_time;dur=0;desc="SQL 0 queries"`
-                data.append(
-                    '{}_{};dur={};desc="{}"'.format(
-                        panel.panel_id, key, record.get("value"), record.get("title")
-                    )
-                )
-
-        if data:
-            response["Server-Timing"] = ", ".join(data)
-        return response
+            for header, value in panel.get_headers(request).items():
+                if header in headers:
+                    headers[header] += f", {value}"
+                else:
+                    headers[header] = value
+        return headers
