@@ -1,20 +1,17 @@
-from __future__ import absolute_import, unicode_literals
-
-import warnings
-
 from django.template.loader import render_to_string
 
 from debug_toolbar import settings as dt_settings
 from debug_toolbar.utils import get_name_from_obj
 
 
-class Panel(object):
+class Panel:
     """
     Base class for panels.
     """
 
-    def __init__(self, toolbar):
+    def __init__(self, toolbar, get_response):
         self.toolbar = toolbar
+        self.get_response = get_response
 
     # Private panel properties
 
@@ -66,6 +63,15 @@ class Panel(object):
         """
         return True
 
+    @property
+    def is_historical(self):
+        """
+        Panel supports rendering historical values.
+
+        Defaults to :attr:`has_content`.
+        """
+        return self.has_content
+
     @property
     def title(self):
         """
@@ -81,7 +87,7 @@ class Panel(object):
         Template used to render :attr:`content`.
 
         Mandatory, unless the panel sets :attr:`has_content` to ``False`` or
-        overrides `attr`:content`.
+        overrides :attr:`content`.
         """
         raise NotImplementedError
 
@@ -97,6 +103,17 @@ class Panel(object):
         if self.has_content:
             return render_to_string(self.template, self.get_stats())
 
+    @property
+    def scripts(self):
+        """
+        Scripts used by the HTML content of the panel when it's displayed.
+
+        When a panel is rendered on the frontend, the ``djdt.panel.render``
+        JavaScript event will be dispatched. The scripts can listen for
+        this event to support dynamic functionality.
+        """
+        return []
+
     # URLs for panel-specific views
 
     @classmethod
@@ -118,8 +135,7 @@ class Panel(object):
         time.
 
         Unless the toolbar or this panel is disabled, this method will be
-        called early in :class:`DebugToolbarMiddleware.process_request`. It
-        should be idempotent.
+        called early in ``DebugToolbarMiddleware``. It should be idempotent.
         """
 
     def disable_instrumentation(self):
@@ -129,8 +145,7 @@ class Panel(object):
         This is the opposite of :meth:`enable_instrumentation`.
 
         Unless the toolbar or this panel is disabled, this method will be
-        called late in :class:`DebugToolbarMiddleware.process_response`. It
-        should be idempotent.
+        called late in the middleware. It should be idempotent.
         """
 
     # Store and retrieve stats (shared between panels for no good reason)
@@ -155,7 +170,7 @@ class Panel(object):
 
         Each call to ``record_stats`` updates the statistics dictionary.
         """
-        data = {key: dict(title=title, value=value)}
+        data = {key: {"title": title, "value": value}}
         self.toolbar.server_timing_stats.setdefault(self.panel_id, {}).update(data)
 
     def get_server_timing_stats(self):
@@ -168,44 +183,50 @@ class Panel(object):
 
     def process_request(self, request):
         """
-        Like process_request in Django's middleware.
+        Like __call__ in Django's middleware.
 
         Write panel logic related to the request there. Save data with
         :meth:`record_stats`.
-        """
 
-    def process_view(self, request, view_func, view_args, view_kwargs):
+        Return the existing response or overwrite it.
         """
-        Like process_view in Django's middleware.
+        return self.get_response(request)
 
-        Write panel logic related to the view there. Save data with
-        :meth:`record_stats`.
+    def get_headers(self, request):
         """
+        Get headers the panel needs to set.
 
-    def process_response(self, request, response):
-        """
-        Like process_response in Django's middleware. This is similar to
-        :meth:`generate_stats <debug_toolbar.panels.Panel.generate_stats>`,
-        but will be executed on every request. It should be used when either
-        the logic needs to be executed on every request or it needs to change
-        the response entirely, such as :class:`RedirectsPanel`.
+        Called after :meth:`process_request
+        <debug_toolbar.panels.Panel.generate_stats>` and
+        :meth:`process_request<debug_toolbar.panels.Panel.generate_stats>`
 
-        Write panel logic related to the response there. Post-process data
-        gathered while the view executed. Save data with :meth:`record_stats`.
+        Header values will be appended if multiple panels need to set it.
 
-        Return a response to overwrite the existing response.
+        By default it sets the Server-Timing header.
+
+        Return dict of headers to be appended.
         """
+        headers = {}
+        stats = self.get_server_timing_stats()
+        if stats:
+            headers["Server-Timing"] = ", ".join(
+                # example: `SQLPanel_sql_time;dur=0;desc="SQL 0 queries"`
+                '{}_{};dur={};desc="{}"'.format(
+                    self.panel_id, key, record.get("value"), record.get("title")
+                )
+                for key, record in stats.items()
+            )
+        return headers
 
     def generate_stats(self, request, response):
         """
-        Similar to :meth:`process_response
-        <debug_toolbar.panels.Panel.process_response>`,
-        but may not be executed on every request. This will only be called if
-        the toolbar will be inserted into the request.
-
         Write panel logic related to the response there. Post-process data
         gathered while the view executed. Save data with :meth:`record_stats`.
 
+        Called after :meth:`process_request
+        <debug_toolbar.panels.Panel.process_request>`.
+
+
         Does not return a value.
         """
 
@@ -219,9 +240,14 @@ class Panel(object):
         Does not return a value.
         """
 
+    @classmethod
+    def run_checks(cls):
+        """
+        Check that the integration is configured correctly for the panel.
 
-# Backward-compatibility for 1.0, remove in 2.0.
-class DebugPanel(Panel):
-    def __init__(self, *args, **kwargs):
-        warnings.warn("DebugPanel was renamed to Panel.", DeprecationWarning)
-        super(DebugPanel, self).__init__(*args, **kwargs)
+        This will be called as a part of the Django checks system when the
+        application is being setup.
+
+        Return a list of :class:`django.core.checks.CheckMessage` instances.
+        """
+        return []
