@@ -3,13 +3,18 @@ import sys
 import time
 from collections import OrderedDict
 
+try:
+    from django.utils.connection import ConnectionProxy
+except ImportError:
+    ConnectionProxy = None
+
 from django.conf import settings
 from django.core import cache
-from django.core.cache import CacheHandler, caches as original_caches
+from django.core.cache import DEFAULT_CACHE_ALIAS, CacheHandler
 from django.core.cache.backends.base import BaseCache
 from django.dispatch import Signal
 from django.middleware import cache as middleware_cache
-from django.utils.translation import gettext_lazy as _, ngettext as __
+from django.utils.translation import gettext_lazy as _, ngettext
 
 from debug_toolbar import settings as dt_settings
 from debug_toolbar.panels import Panel
@@ -20,9 +25,7 @@ from debug_toolbar.utils import (
     tidy_stacktrace,
 )
 
-cache_called = Signal(
-    providing_args=["time_taken", "name", "return_value", "args", "kwargs", "trace"]
-)
+cache_called = Signal()
 
 
 def send_signal(method):
@@ -60,7 +63,7 @@ class CacheStatTracker(BaseCache):
         self.cache = cache
 
     def __repr__(self):
-        return str("<CacheStatTracker for %s>") % repr(self.cache)
+        return "<CacheStatTracker for %s>" % repr(self.cache)
 
     def _get_func_info(self):
         frame = sys._getframe(3)
@@ -85,6 +88,14 @@ class CacheStatTracker(BaseCache):
     def set(self, *args, **kwargs):
         return self.cache.set(*args, **kwargs)
 
+    @send_signal
+    def get_or_set(self, *args, **kwargs):
+        return self.cache.get_or_set(*args, **kwargs)
+
+    @send_signal
+    def touch(self, *args, **kwargs):
+        return self.cache.touch(*args, **kwargs)
+
     @send_signal
     def delete(self, *args, **kwargs):
         return self.cache.delete(*args, **kwargs)
@@ -97,7 +108,7 @@ class CacheStatTracker(BaseCache):
     def has_key(self, *args, **kwargs):
         # Ignore flake8 rules for has_key since we need to support caches
         # that may be using has_key.
-        return self.cache.has_key(*args, **kwargs)  # noqa
+        return self.cache.has_key(*args, **kwargs)  # noqa: W601
 
     @send_signal
     def incr(self, *args, **kwargs):
@@ -129,9 +140,16 @@ class CacheStatTracker(BaseCache):
 
 
 class CacheHandlerPatch(CacheHandler):
-    def __getitem__(self, alias):
-        actual_cache = super().__getitem__(alias)
-        return CacheStatTracker(actual_cache)
+    def __init__(self, settings=None):
+        self._djdt_wrap = True
+        super().__init__(settings=settings)
+
+    def create_connection(self, alias):
+        actual_cache = super().create_connection(alias)
+        if self._djdt_wrap:
+            return CacheStatTracker(actual_cache)
+        else:
+            return actual_cache
 
 
 middleware_cache.caches = CacheHandlerPatch()
@@ -155,6 +173,8 @@ class CachePanel(Panel):
                 ("add", 0),
                 ("get", 0),
                 ("set", 0),
+                ("get_or_set", 0),
+                ("touch", 0),
                 ("delete", 0),
                 ("clear", 0),
                 ("get_many", 0),
@@ -180,9 +200,9 @@ class CachePanel(Panel):
         trace=None,
         template_info=None,
         backend=None,
-        **kw
+        **kw,
     ):
-        if name == "get":
+        if name == "get" or name == "get_or_set":
             if return_value is None:
                 self.misses += 1
             else:
@@ -216,39 +236,42 @@ class CachePanel(Panel):
     @property
     def nav_subtitle(self):
         cache_calls = len(self.calls)
-        return (
-            __(
-                "%(cache_calls)d call in %(time).2fms",
-                "%(cache_calls)d calls in %(time).2fms",
-                cache_calls,
-            )
-            % {"cache_calls": cache_calls, "time": self.total_time}
-        )
+        return ngettext(
+            "%(cache_calls)d call in %(time).2fms",
+            "%(cache_calls)d calls in %(time).2fms",
+            cache_calls,
+        ) % {"cache_calls": cache_calls, "time": self.total_time}
 
     @property
     def title(self):
         count = len(getattr(settings, "CACHES", ["default"]))
-        return (
-            __(
-                "Cache calls from %(count)d backend",
-                "Cache calls from %(count)d backends",
-                count,
-            )
-            % {"count": count}
-        )
+        return ngettext(
+            "Cache calls from %(count)d backend",
+            "Cache calls from %(count)d backends",
+            count,
+        ) % {"count": count}
 
     def enable_instrumentation(self):
-        if isinstance(middleware_cache.caches, CacheHandlerPatch):
-            cache.caches = middleware_cache.caches
-        else:
-            cache.caches = CacheHandlerPatch()
+        for alias in cache.caches:
+            if not isinstance(cache.caches[alias], CacheStatTracker):
+                cache.caches[alias] = CacheStatTracker(cache.caches[alias])
+
+        if not isinstance(middleware_cache.caches, CacheHandlerPatch):
+            middleware_cache.caches = cache.caches
+
+        # Wrap the patched cache inside Django's ConnectionProxy
+        if ConnectionProxy:
+            cache.cache = ConnectionProxy(cache.caches, DEFAULT_CACHE_ALIAS)
 
     def disable_instrumentation(self):
-        cache.caches = original_caches
+        for alias in cache.caches:
+            if isinstance(cache.caches[alias], CacheStatTracker):
+                cache.caches[alias] = cache.caches[alias].cache
+        if ConnectionProxy:
+            cache.cache = ConnectionProxy(cache.caches, DEFAULT_CACHE_ALIAS)
         # While it can be restored to the original, any views that were
         # wrapped with the cache_page decorator will continue to use a
         # monkey patched cache.
-        middleware_cache.caches = original_caches
 
     def generate_stats(self, request, response):
         self.record_stats(
