@@ -1,11 +1,9 @@
 import json
-import logging
-import sys
 from collections import OrderedDict
 
-from django.conf import settings
 from django.http.request import RawPostDataException
 from django.template.loader import render_to_string
+from django.templatetags.static import static
 from django.urls import path
 from django.utils import timezone
 from django.utils.translation import gettext_lazy as _
@@ -14,16 +12,28 @@ from debug_toolbar.panels import Panel
 from debug_toolbar.panels.history import views
 from debug_toolbar.panels.history.forms import HistoryStoreForm
 
-logger = logging.getLogger(__name__)
-
 
 class HistoryPanel(Panel):
-    """ A panel to display History """
+    """A panel to display History"""
 
     title = _("History")
     nav_title = _("History")
     template = "debug_toolbar/panels/history.html"
 
+    def get_headers(self, request):
+        headers = super().get_headers(request)
+        observe_request = self.toolbar.get_observe_request()
+        store_id = getattr(self.toolbar, "store_id")
+        if store_id and observe_request(request):
+            headers["DJDT-STORE-ID"] = store_id
+        return headers
+
+    @property
+    def enabled(self):
+        # Do not show the history panel if the panels are rendered on request
+        # rather than loaded via ajax.
+        return super().enabled and not self.toolbar.should_render_panels()
+
     @property
     def is_historical(self):
         """The HistoryPanel should not be included in the historical panels."""
@@ -53,14 +63,10 @@ class HistoryPanel(Panel):
                 and request.body
                 and request.META.get("CONTENT_TYPE") == "application/json"
             ):
-                # Python <= 3.5's json.loads expects a string.
-                data = json.loads(
-                    request.body
-                    if sys.version_info[:2] > (3, 5)
-                    else request.body.decode(
-                        request.encoding or settings.DEFAULT_CHARSET
-                    )
-                )
+                try:
+                    data = json.loads(request.body)
+                except ValueError:
+                    pass
         except RawPostDataException:
             # It is not guaranteed that we may read the request data (again).
             data = None
@@ -69,6 +75,7 @@ class HistoryPanel(Panel):
             {
                 "request_url": request.get_full_path(),
                 "request_method": request.method,
+                "status_code": response.status_code,
                 "data": data,
                 "time": timezone.now(),
             }
@@ -84,7 +91,9 @@ class HistoryPanel(Panel):
         for id, toolbar in reversed(self.toolbar._store.items()):
             stores[id] = {
                 "toolbar": toolbar,
-                "form": HistoryStoreForm(initial={"store_id": id}),
+                "form": HistoryStoreForm(
+                    initial={"store_id": id, "exclude_history": True}
+                ),
             }
 
         return render_to_string(
@@ -93,7 +102,16 @@ class HistoryPanel(Panel):
                 "current_store_id": self.toolbar.store_id,
                 "stores": stores,
                 "refresh_form": HistoryStoreForm(
-                    initial={"store_id": self.toolbar.store_id}
+                    initial={
+                        "store_id": self.toolbar.store_id,
+                        "exclude_history": True,
+                    }
                 ),
             },
         )
+
+    @property
+    def scripts(self):
+        scripts = super().scripts
+        scripts.append(static("debug_toolbar/js/history.js"))
+        return scripts
