@@ -1,13 +1,13 @@
-from __future__ import absolute_import, unicode_literals
-
 from collections import OrderedDict
 
 from django.conf import settings
-from django.utils.translation import ugettext_lazy as _
-from django.views.debug import get_safe_settings
+from django.utils.translation import gettext_lazy as _
+from django.views.debug import get_default_exception_reporter_filter
 
 from debug_toolbar.panels import Panel
 
+get_safe_settings = get_default_exception_reporter_filter().get_safe_settings
+
 
 class SettingsPanel(Panel):
     """
@@ -19,7 +19,7 @@ class SettingsPanel(Panel):
     nav_title = _("Settings")
 
     def title(self):
-        return _("Settings from <code>%s</code>") % settings.SETTINGS_MODULE
+        return _("Settings from %s") % settings.SETTINGS_MODULE
 
     def generate_stats(self, request, response):
         self.record_stats(
