@@ -2,29 +2,36 @@
 The main DebugToolbar class that loads and renders the Toolbar.
 """
 
-from __future__ import absolute_import, unicode_literals
-
 import uuid
 from collections import OrderedDict
+from functools import lru_cache
 
 from django.apps import apps
-from django.conf.urls import url
 from django.core.exceptions import ImproperlyConfigured
 from django.template import TemplateSyntaxError
 from django.template.loader import render_to_string
+from django.urls import path, resolve
+from django.urls.exceptions import Resolver404
 from django.utils.module_loading import import_string
 
-from debug_toolbar import settings as dt_settings
+from debug_toolbar import APP_NAME, settings as dt_settings
 
 
-class DebugToolbar(object):
-    def __init__(self, request):
+class DebugToolbar:
+    def __init__(self, request, get_response):
         self.request = request
         self.config = dt_settings.get_config().copy()
+        panels = []
+        for panel_class in reversed(self.get_panel_classes()):
+            panel = panel_class(self, get_response)
+            panels.append(panel)
+            if panel.enabled:
+                get_response = panel.process_request
+        self.process_request = get_response
         self._panels = OrderedDict()
-        for panel_class in self.get_panel_classes():
-            panel_instance = panel_class(self)
-            self._panels[panel_instance.panel_id] = panel_instance
+        while panels:
+            panel = panels.pop()
+            self._panels[panel.panel_id] = panel
         self.stats = {}
         self.server_timing_stats = {}
         self.store_id = None
@@ -72,27 +79,28 @@ class DebugToolbar(object):
             else:
                 raise
 
-    # Handle storing toolbars in memory and fetching them later on
-
-    _store = OrderedDict()
-
     def should_render_panels(self):
+        """Determine whether the panels should be rendered during the request
+
+        If False, the panels will be loaded via Ajax.
+        """
         render_panels = self.config["RENDER_PANELS"]
         if render_panels is None:
             render_panels = self.request.META["wsgi.multiprocess"]
         return render_panels
 
+    # Handle storing toolbars in memory and fetching them later on
+
+    _store = OrderedDict()
+
     def store(self):
+        # Store already exists.
+        if self.store_id:
+            return
         self.store_id = uuid.uuid4().hex
-        cls = type(self)
-        cls._store[self.store_id] = self
-        for _ in range(len(cls._store) - self.config["RESULTS_CACHE_SIZE"]):
-            try:
-                # collections.OrderedDict
-                cls._store.popitem(last=False)
-            except TypeError:
-                # django.utils.datastructures.SortedDict
-                del cls._store[cls._store.keyOrder[0]]
+        self._store[self.store_id] = self
+        for _ in range(self.config["RESULTS_CACHE_SIZE"], len(self._store)):
+            self._store.popitem(last=False)
 
     @classmethod
     def fetch(cls, store_id):
@@ -123,7 +131,7 @@ class DebugToolbar(object):
             # Load URLs in a temporary variable for thread safety.
             # Global URLs
             urlpatterns = [
-                url(r"^render_panel/$", views.render_panel, name="render_panel")
+                path("render_panel/", views.render_panel, name="render_panel"),
             ]
             # Per-panel URLs
             for panel_class in cls.get_panel_classes():
@@ -131,6 +139,35 @@ class DebugToolbar(object):
             cls._urlpatterns = urlpatterns
         return cls._urlpatterns
 
+    @classmethod
+    def is_toolbar_request(cls, request):
+        """
+        Determine if the request is for a DebugToolbar view.
+        """
+        # The primary caller of this function is in the middleware which may
+        # not have resolver_match set.
+        try:
+            resolver_match = request.resolver_match or resolve(
+                request.path, getattr(request, "urlconf", None)
+            )
+        except Resolver404:
+            return False
+        return resolver_match.namespaces and resolver_match.namespaces[-1] == APP_NAME
 
-app_name = "djdt"
-urlpatterns = DebugToolbar.get_urls()
+    @staticmethod
+    @lru_cache(maxsize=128)
+    def get_observe_request():
+        # If OBSERVE_REQUEST_CALLBACK is a string, which is the recommended
+        # setup, resolve it to the corresponding callable.
+        func_or_path = dt_settings.get_config()["OBSERVE_REQUEST_CALLBACK"]
+        if isinstance(func_or_path, str):
+            return import_string(func_or_path)
+        else:
+            return func_or_path
+
+
+def observe_request(request):
+    """
+    Determine whether to update the toolbar from a client side request.
+    """
+    return not DebugToolbar.is_toolbar_request(request)
