@@ -3,12 +3,12 @@ import os.path
 import re
 import sys
 from importlib import import_module
-from itertools import chain
+from pprint import pformat
 
 import django
 from django.core.exceptions import ImproperlyConfigured
 from django.template import Node
-from django.template.loader import render_to_string
+from django.utils.html import format_html
 from django.utils.safestring import mark_safe
 
 from debug_toolbar import settings as dt_settings
@@ -27,7 +27,7 @@ def get_module_path(module_name):
     try:
         module = import_module(module_name)
     except ImportError as e:
-        raise ImproperlyConfigured("Error importing HIDE_IN_STACKTRACES: {}".format(e))
+        raise ImproperlyConfigured(f"Error importing HIDE_IN_STACKTRACES: {e}")
     else:
         source_path = inspect.getsourcefile(module)
         if source_path.endswith("__init__.py"):
@@ -69,26 +69,38 @@ def tidy_stacktrace(stack):
 
 
 def render_stacktrace(trace):
-    stacktrace = []
-    for frame in trace:
-        params = (v for v in chain(frame[0].rsplit(os.path.sep, 1), frame[1:]))
-        params_dict = {str(idx): v for idx, v in enumerate(params)}
-        try:
-            stacktrace.append(params_dict)
-        except KeyError:
-            # This frame doesn't have the expected format, so skip it and move
-            # on to the next one
-            continue
-
-    return mark_safe(
-        render_to_string(
-            "debug_toolbar/panels/sql_stacktrace.html",
-            {
-                "stacktrace": stacktrace,
-                "show_locals": dt_settings.get_config()["ENABLE_STACKTRACES_LOCALS"],
-            },
+    show_locals = dt_settings.get_config()["ENABLE_STACKTRACES_LOCALS"]
+    html = ""
+    for abspath, lineno, func, code, locals_ in trace:
+        if os.path.sep in abspath:
+            directory, filename = abspath.rsplit(os.path.sep, 1)
+            # We want the separator to appear in the UI so add it back.
+            directory += os.path.sep
+        else:
+            # abspath could be something like "<frozen importlib._bootstrap>"
+            directory = ""
+            filename = abspath
+        html += format_html(
+            (
+                '<span class="djdt-path">{}</span>'
+                + '<span class="djdt-file">{}</span> in'
+                + ' <span class="djdt-func">{}</span>'
+                + '(<span class="djdt-lineno">{}</span>)\n'
+                + '  <span class="djdt-code">{}</span>\n'
+            ),
+            directory,
+            filename,
+            func,
+            lineno,
+            code,
         )
-    )
+        if show_locals:
+            html += format_html(
+                '  <pre class="djdt-locals">{}</pre>\n',
+                pformat(locals_),
+            )
+        html += "\n"
+    return mark_safe(html)
 
 
 def get_template_info():
@@ -135,7 +147,14 @@ def get_template_context(node, context, context_lines=3):
 
 
 def get_template_source_from_exception_info(node, context):
-    exception_info = context.template.get_exception_info(Exception("DDT"), node.token)
+    if context.template.origin == node.origin:
+        exception_info = context.template.get_exception_info(
+            Exception("DDT"), node.token
+        )
+    else:
+        exception_info = context.render_context.template.get_exception_info(
+            Exception("DDT"), node.token
+        )
     line = exception_info["line"]
     source_lines = exception_info["source_lines"]
     name = exception_info["name"]
@@ -150,7 +169,7 @@ def get_name_from_obj(obj):
 
     if hasattr(obj, "__module__"):
         module = obj.__module__
-        name = "{}.{}".format(module, name)
+        name = f"{module}.{name}"
 
     return name
 
@@ -198,7 +217,7 @@ def getframeinfo(frame, context=1):
         for line in first_lines[:2]:
             # File coding may be specified. Match pattern from PEP-263
             # (https://www.python.org/dev/peps/pep-0263/)
-            match = re.search(br"coding[:=]\s*([-\w.]+)", line)
+            match = re.search(rb"coding[:=]\s*([-\w.]+)", line)
             if match:
                 encoding = match.group(1).decode("ascii")
                 break
@@ -210,6 +229,16 @@ def getframeinfo(frame, context=1):
         return (filename, lineno, frame.f_code.co_name, lines, index)
 
 
+def get_sorted_request_variable(variable):
+    """
+    Get a sorted list of variables from the request data.
+    """
+    if isinstance(variable, dict):
+        return [(k, variable.get(k)) for k in sorted(variable)]
+    else:
+        return [(k, variable.getlist(k)) for k in sorted(variable)]
+
+
 def get_stack(context=1):
     """
     Get a list of records for a frame and all higher (calling) frames.
@@ -242,14 +271,14 @@ class ThreadCollector:
         is provided, returns a list for the current thread.
         """
         if thread is None:
-            thread = threading.currentThread()
+            thread = threading.current_thread()
         if thread not in self.collections:
             self.collections[thread] = []
         return self.collections[thread]
 
     def clear_collection(self, thread=None):
         if thread is None:
-            thread = threading.currentThread()
+            thread = threading.current_thread()
         if thread in self.collections:
             del self.collections[thread]
 
